
% ******************************************************************************
% Week B Script to compute state-feedback gains for a single-input single-output 
% third-order system, with possibly an additional integral state.
%
% Input: matrices A(3x3), B(3x1), C(1x3) and D(1x1) for a system described by
%     d/dt x(t) = Ax(t) + Bu(t), y(t) = Cx(t) + Du(t)%
% 
% Output: K(1x3) and Kr, state gains and input scaling or
%         K(1x4) where K(4) is the gain on the integral state.
%
% K's are determined using pole placement or LQR
% ******************************************************************************

close all;
clearvars;

% A matrix, multiplies x
A = zeros(3);
% B matrix multiplies the scalar input u
B = zeros(3,1);
% C matrix, multiplies x
C = zeros(1,3);
% D matrix multiplies u, scalar, no preallocation.

% --------------------------------------------------------------------
% TODO (replace with code from last week): define the A and B matrices
% --------------------------------------------------------------------
% First Row
A(1,1) = 0.0;
A(1,2) = 0.0;
A(1,3) = 0.0;
B(1)   = 0.0;
s1     = 'icoil';
% Second Row
A(2,1) = 0.0;
A(2,2) = 0.0;
A(2,3) = 0.0;
B(2)   = 0.0;
s2     = 'vel';
% Third Row
A(3,1) = 0.0;
A(3,2) = 0.0;
A(3,3) = 0.0;
B(3)   = 0.0;
s3     = 'y';


% ------------------------------------------------------------
% TODO (replace with code from last week): define the C matrix
% ------------------------------------------------------------
% Fill in the output matrix
C(1) = 0.0;
C(2) = 0.0;
C(3) = 0.0;

% The instantaneous u-to-y term (D) is zero.
D = 0;

% Convert to a system data structure
sys = ss(A, B, C, D, 'StateName', {s1,s2,s3});


% ----------------------
% TODO (Checkoff 4): LQR
% ----------------------
% Select state weights Q, and control weights R 
Q = diag([0.0,0.0,0.0]);
R = 0;
Klqr = lqr(sys,Q,R);

K = Klqr; fprintf('lqr controller selected\n')

% Print input-output transfer function.
tfsys = tf(sys);
if tfsys.num{1}(end) ~= 0
    fprintf('Open loop system')
    display(tfsys);
end

% Determine poles
ps = pole(sys);
fprintf('Open-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));

% ----------------------------------------------------------------
% TODO (replace with code from last week at first
%       but you will need to change for Checkoff 5): Define Kr
% ----------------------------------------------------------------
Kr = ??;


% --------------------------------------------------------------------
% TODO (replace with code from last week): Define matrices Acl and Bcl
% --------------------------------------------------------------------
Acl = ??;  
Bcl = ??;

% Recall that u = Kr*y_d - K*x for the open loop system, and that for the 
% closed-loop system described by Acl,Bcl,Ccl,Dcl matrices, 
% dx/dt = Acl*x + Bcl*y_d and y = Ccl*x + Dcl*y_d

% The Cplot and Dplot matrices below are set so that 
% the second through fourth elements of Cplot*x + Dplot*y_d are the three 
% states. We replace the zero elements in Cplot and Dplot below so that the 
% first element of Cplot*x + Dplot*y_d is equal to the command,
% u =  = Kr*y_d - K*x
Cplot = [-K; eye(3)]; 
Dplot = [Kr; zeros(3,1);]; 

% Create the closed loop systen
sys_cl = ss(Acl, Bcl, Cplot, Dplot,'OutputName', {'cmd', s1, s2, s3});
ps = pole(sys_cl);
fprintf('Closed-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));

% Plot step response of the command and all three states 
figure;
step(sys_cl, 0.5); grid;
title(sprintf("Step responses for state space (no integrator)"));
fprintf('K%s=%f, K%s=%f, K%s=%f, Kr=%f\n', s1,K(1),s2,K(2),s3,K(3), Kr);

% ******************************************************************
% Checkoff 5: Now let's add in the integrator, while still using LQR
%             Make sure to remove the block comment, %{ and %}
% ******************************************************************


% augment the original equations
% p indicates matrices for controller with an integral
Bp = [0; B];
Cp = [0 C];
Dp = D;
Ap = zeros(4);
Ap(2:4,:) = [zeros(3,1) A]; % we set the last 3 rows of Ap for you

% ------------------------------
% TODO (Checkoff 5): Complete Ap
% ------------------------------
Ap(1, :) = [0 0 0 0];

% Convert to a system (lqr design ignores Br)
sysp = ss(Ap, Bp, Cp, Dp, 'StateName',{'Int', s1, s2, s3});

% ----------------------------------------------------------
% TODO (Checkoff 5): Complete Qp and Rp
% ----------------------------------------------------------
Qp = diag([0.0,0.0,0.0,0.0]);
Rp = diag(0.0);

Kp = lqr(sysp,Qp,Rp);

% ----------------------------------------------------------
% TODO (Checkoff 5): Complete Aclp and Bclp
% ----------------------------------------------------------
Aclp = ??; 
Bclp = [0.0;0.0;0.0;0.0];

Cplotp = [Kp; eye(4);];
Dplotp = [0.0; zeros(4,1)]; 

% Closed Loop systen
sys_clp = ss(Aclp, Bclp, Cplotp, Dplotp, ...
'OutputName', {'cmd','Integral',s1,s2,s3});
ps = pole(sys_clp);
fprintf('Closed-Loop Poles:%f,%f,%f\n', ps(1),ps(2),ps(3));

% Plot step response
figure;
step(sys_clp,0.5); grid;
title(sprintf("Step responses for state space (with integrator)"));
fprintf('Kint=%f K%s=%f, K%s=%f, K%s=%f, \n',Kp(1),s1,Kp(2),s2,Kp(3),s3,Kp(4));
fprintf('Done With Integrator\n');


